from mcsniperpy.util.request_manager import RequestManager
from mcsniperpy.util.logs_manager import Logger as log
from mcsniperpy.util.logs_manager import Color as color


async def announce(
    username: str,
    authorization: str,
    session: RequestManager,
    full_url: str = "https://announcements-api.herokuapp.com/api/v1/announce",
) -> bool:
    headers = {"Authorization": authorization}
    json = {"name": username}
    resp, resp_json, _ = await session.post(full_url, json=json, headers=headers)
    if resp.status < 300:
        log.info(f'{color.white}[{color.l_green}success{color.white}]{color.reset}')
        return True

    log.error(f"Failed to announce snipe | {resp.status} | {resp_json}")
    return False




import asyncio
import ssl
import time

from mcsniperpy.util.logs_manager import Color as color
from mcsniperpy.util.logs_manager import Logger as log


class Account:
    def __init__(self, email, password, *sqs, **kwargs):
        if kwargs.get('acc_type') == 'microsoft':
            log.error('MCsniperPY does not support Microsoft accounts at the moment.')
        self.email = email  # email
        self.password = password  # password
        self.sqs = sqs  # security question answers

        self.questions = []  # Actual questions
        self.answer_ids = []  # IDS of the security question answers

        self.username = ""  # Totally useless lol

        self.bearer = ""
        self.headers = {"Authorization": "Bearer TOKEN"}

        self.snipe_data = ("PUT /minecraft/profile/name/NAME HTTP/1.1\r\n"
                           "Host: api.minecraftservices.com\r\n"
                           "Authorization: Bearer TOKEN\r\n"
                           "\r\n").encode()

        self.readers_writers = []

    def encode_snipe_data(self, name):
        self.snipe_data = (f"PUT /minecraft/profile/name/{name} HTTP/1.1\r\n"
                           "Host: api.minecraftservices.com\r\n"
                           f"Authorization: Bearer {self.bearer}\r\n"
                           "\r\n").encode()

    def encode_snipe_data_proxy(self, name):
        self.snipe_data = (f"PUT https://api.minecraftservices.com/minecraft/profile/name/{name} HTTP/1.1\r\n"
                           "Host: api.minecraftservices.com\r\n"
                           f"Authorization: Bearer {self.bearer}\r\n"
                           "\r\n").encode()

    async def authenticate(self, session) -> bool:
        resp, _, resp_json = await session.post(
            "https://authserver.mojang.com/authenticate",
            json={
                "agent": {
                    "name": "Minecraft",
                    "version": 1
                },
                "username": self.email,
                "password": self.password,
                "requestUser": "false"
            },
            headers={"Content-Type": "application/json"}
        )

        if resp.status == 200:
            self.headers["Authorization"] = f"Bearer {resp_json['accessToken']}"  # authorization header
            self.bearer = resp_json['accessToken']
            return True
        return False

    async def get_questions(self, session) -> None:
        resp, _, resp_json = await session.get(
            "https://api.mojang.com/user/security/challenges",
            headers=self.headers
        )

        self.answer_ids = [question["answer"]["id"] for question in resp_json]
        self.questions = [question["question"]["question"] for question in resp_json]

        if resp.status == 200:
            return
        log.error(f"failed to get security questions for {self.email}")

    async def need_sqs(self, session) -> bool:
        resp, _, _ = await session.get(
            "https://api.mojang.com/user/security/location",
            headers=self.headers
        )

        return resp.status == 403

    async def submit_questions(self, session):

        if self.sqs == ():
            log.error(f"security questions for {self.email} not provided!")
            return False

        resp, _, _ = await session.post(
            "https://api.mojang.com/user/security/location",
            headers=self.headers,
            json=[
                {
                    "id": self.answer_ids[0],
                    "answer": self.sqs[0]
                },
                {
                    "id": self.answer_ids[1],
                    "answer": self.sqs[1]
                },
                {
                    "id": self.answer_ids[2],
                    "answer": self.sqs[2]
                }
            ]
        )

        # Yes, that's kinda ugly lol. whatever.

        if resp.status == 204:
            return True
        log.error(f"security questions for {self.email} are incorrect!")
        return False

    async def fully_authenticate(self, session):
        if not await self.authenticate(session):
            log.error(f"failed to auth {self.email}")
            return

        if not await self.need_sqs(session):
            log.info(f"{color.white}[{color.l_green}success{color.white}]{color.reset} authenticated {self.email}")
        else:
            await self.get_questions(session)
            if await self.submit_questions(session):
                log.info(f"{color.white}[{color.l_green}success{color.white}]{color.reset} authed {self.email} with security questions")
            else:
                log.error(f"failed to authenticate {self.email}")

    async def snipe_connect(self) -> None:

        reader, writer = await asyncio.open_connection("api.minecraftservices.com", 443, ssl=ssl.SSLContext(),
                                                       ssl_handshake_timeout=5)
        log.debug(f"Connected on account {self.email}")
        writer.write(self.snipe_data[0:-2])

        await writer.drain()
        self.readers_writers.append((reader, writer))

    async def snipe(self, writer: asyncio.StreamWriter, do_log: bool = True) -> None:
        writer.write(self.snipe_data[-2:])
        await writer.drain()
        if do_log:
            log.info(f"sent request @ {time.time()}")
        # Simpler code 🔽 encompasses all of these snipe* functions
        # async with snipe_session.put("https://api.minecraftservices.com/minecraft/profile/name/%s" % "blah",
        #                              headers={
        #                                  "Authorization": "Bearer %s" % self.bearer, "Content-Type": "application/json
        # ^^^^ THAT LINE IS INVALID YOU NEED A " AT THE END ^^^
        #                              }) as r:
        #     print("[%s] %s @ %5f" % (name, r.status, time.time()))
        #     if r.status == 204:
        #         return True, self.email
        #     else:
        #         return False, None

    async def snipe_read(
            self, name: str, reader: asyncio.StreamReader, writer: asyncio.StreamWriter, do_log: bool = True) -> (
            bool, str, float):
        resp = await reader.read(12)
        now = time.time()
        status = int(resp[9:12])
        is_success = status < 300

        writer.close()
        await writer.wait_closed()

        if do_log:
            pretty_status = '%s%s%s' % (
                {
                    False: color.l_red,
                    True: color.l_green
                }.get(is_success, color.l_red),
                status,
                color.reset
            )
            pretty_name = '%s%s%s' % (
                color.l_cyan,
                name,
                color.reset
            )

            log.info("[%s] [%s] @ %.10f" % (pretty_name, pretty_status, now))
        return is_success, self.email, now



import asyncio
import os.path
import time

import aiohttp

from mcsniperpy.util import request_manager
from mcsniperpy.util import utils as util
from mcsniperpy.util.classes.config import BackConfig, populate_configs, Config
from mcsniperpy.util.name_system import api_timing, namemc_timing
from mcsniperpy.util import announce

class Sniper:
    def __init__(self,
                 colorer,
                 logger):
        self.color = colorer
        self.log = logger
        self.session = request_manager.RequestManager(
            None  # aiohttp.ClientSession(connector=aiohttp.TCPConnector(limit=300),headers={})
        )

        self.target = str()  # target username
        self.offset = int()  # Time offset (e.g., 400 = snipe the name 400ms early)

        self.config = None  # util.classes.config.BackConfig
        self.user_config = None  # util.classes.config.Config

        self.data = None

        self.accounts = []  # list of Accounts

    @property
    def initialized(self):
        return self.config.config['sniper'].get('init_path', '') != ''

    @staticmethod
    def init() -> None:

        populate_configs()

    async def run(self, target=None, offset=None):

        self.session.session = aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(limit=300),
            headers={}
        )

        self.config = BackConfig()
        self.log.debug(f"Using sniping path of {self.config.config['sniper'].get('init_path')}")

        self.user_config = Config(os.path.join(self.config.config['sniper'].get('init_path'), 'config.ini'))

        if target is None:
            self.log.debug('No username detected')
            target = self.log.input("Target Username:")
            if target is None:
                self.log.error(f'invalid input "{target}".')
        else:
            self.log.info(f"Sniping username: {target}")

        if offset is None:
            self.log.debug('no offset detected')
            offset = self.log.input("Time Offset:")
            if offset is None or not util.is_float(offset):
                self.log.error(f'Invalid offset input: "{offset}"')
                util.close(1)
            else:
                offset = float(offset)
        else:
            self.log.info(f"Offset (ms): {offset}")

        self.accounts = util.parse_accs(os.path.join(self.config.config['sniper'].get('init_path'), "accounts.txt"))

        timing_system = self.user_config.config['sniper'].get('timing_system', 'kqzz_api').lower()
        start_auth = self.user_config.config['accounts'].getint('start_authentication', '720') * 60
        do_announce = self.user_config.config['announce'].getboolean('announce_snipe', 'no')
        announce_code = self.user_config.config['announce'].getboolean('announce_code', '')

        if timing_system == 'kqzz_api':
            droptime = await api_timing(target, self.session)
        elif timing_system == 'namemc':
            droptime = await namemc_timing(target, self.session)
        else:
            droptime = await api_timing(target, self.session)

        req_count = self.user_config.config['sniper'].getint('snipe_requests', '3')

        await self.snipe(droptime, target, offset, req_count, start_auth, do_announce, announce_code)

    async def snipe(self, droptime, target, offset, req_count, start_auth, do_announce, announce_code):

        authentication_coroutines = [acc.fully_authenticate(session=self.session) for acc in self.accounts]
        pre_snipe_coroutines = [acc.snipe_connect() for _ in range(req_count) for acc in self.accounts]  # For later use

        now = time.time()
        time_until_authentication = 0 if now > (droptime - start_auth) else (droptime - start_auth) - now

        self.log.debug(f'authorizing accounts in {time_until_authentication} seconds.')

        await asyncio.sleep(time_until_authentication)

        await asyncio.gather(*authentication_coroutines)

        for acc in self.accounts:
            acc.encode_snipe_data(target)

        now = time.time()
        time_until_connect = 0 if now > (droptime - 20) else (droptime - 20) - now

        self.log.debug(f'Connecting in {time_until_connect} seconds.')

        await asyncio.sleep(time_until_connect)

        await asyncio.gather(*pre_snipe_coroutines)

        snipe_coroutines = [acc.snipe(acc.readers_writers[i][1]) for i in range(req_count) for acc in self.accounts]

        while time.time() < droptime - offset / 1000:
            await asyncio.sleep(0.00001)  # bad timing solution

        await asyncio.gather(*snipe_coroutines)  # Sends the snipe requests
        responses = await asyncio.gather(
            *[acc.snipe_read(target, acc.readers_writers[i][0], acc.readers_writers[i][1]
                             ) for i in range(req_count) for acc in self.accounts]
        )  # Reads the responses

        for is_success, email, _ in responses:
            if is_success:
                success_acc = util.find_acc_by_email(email, self.accounts)
                self.log.info(f'{self.color.white}[{self.color.l_green}success{self.color.white}]{self.color.reset} '
                              f'sniped {self.target} onto {success_acc.email}')
                if do_announce:
                    await announce.announce(
                        username=target,
                        authorization=announce_code,
                        session=self.session
                    )

    def on_shutdown(self):
        if self.session.session is not None:
            asyncio.run(self.session.session.close())
        if self.log is not None:
            self.log.shutdown()



import asyncio
import shutil

import typer

from mcsniperpy import Sniper
from mcsniperpy.util import logs_manager as log
from mcsniperpy.util import ping_tester
from mcsniperpy.util.name_system import next_name
from mcsniperpy.util.offset_calculator import OffsetCalculator

app = typer.Typer()


def startup():
    width = shutil.get_terminal_size().columns
    print(f"""{log.Color.cyan}
    ███╗   ███╗ ██████╗███████╗███╗   ██╗██╗██████╗ ███████╗██████╗ ██████╗ ██╗   ██╗
    ████╗ ████║██╔════╝██╔════╝████╗  ██║██║██╔══██╗██╔════╝██╔══██╗██╔══██╗╚██╗ ██╔╝
    ██╔████╔██║██║     ███████╗██╔██╗ ██║██║██████╔╝█████╗  ██████╔╝██████╔╝ ╚████╔╝
    ██║╚██╔╝██║██║     ╚════██║██║╚██╗██║██║██╔═══╝ ██╔══╝  ██╔══██╗██╔═══╝   ╚██╔╝
    ██║ ╚═╝ ██║╚██████╗███████║██║ ╚████║██║██║     ███████╗██║  ██║██║        ██║
    ╚═╝     ╚═╝ ╚═════╝╚══════╝╚═╝  ╚═══╝╚═╝╚═╝     ╚══════╝╚═╝  ╚═╝╚═╝        ╚═╝
    """.center(width))
    print(f"{log.Color.cyan}Created by Kqzz#0001")
    print(f"Git: github.com/MCSniperPY | Discord: https://mcsniperpy.github.io/discord{log.Color.white}")


sniper = Sniper(
    log.Color,
    log.Logger
)


@app.command()
def snipe(username: str = typer.Option(None, help="The username to attempt a snipe on"),
          offset: int = typer.Option(None, help="The offset you want to use for the target username."),
          debug: bool = typer.Option(False, help="Enable debug mode."),
          color: bool = typer.Option(True, help="Colored terminal output"),
          next_sorry: int = typer.Option(None, '--next', help='Snipe the next available username.'
                                         ' It is not recommended to use this!')):
    """
    Snipe a minecraft username!
    """
    if debug:
        sniper.log.debug_enabled = True

    if not color:
        sniper.color.disable()

    if next_sorry is not None:
        username = next_name(searches=next_sorry)

    startup()

    asyncio.get_event_loop().run_until_complete(sniper.run(username, offset))
    sniper.on_shutdown()


@app.command()
def ping(iterations: int = typer.Option(5, help='How many times to ping Mojang\'s servers.')):
    """
    Test your ping to Mojang's servers
    """
    asyncio.get_event_loop().run_until_complete(ping_tester.ping_test(iterations))
    sniper.on_shutdown()


@app.command()
def offset_test(accuracy: int = typer.Option(20, help="How accurate do you want your offset to be?"),
                aim_for: float = typer.Option(.15, help="What time do you want to aim for?")):
    """
    Find the optimal offset for you. Note: this is not entirely accurate since it does not compensate for api lag caused\
 by other snipers
    """
    offset_calc = OffsetCalculator(accuracy=accuracy, aim_for=aim_for)
    asyncio.get_event_loop().run_until_complete(offset_calc.run())


@app.command()
def init():
    """
    Initialize MCsniperPY to be able to snipe names. This is an essential step before sniping. Please read the docs for\
 more info. https://docs.mcsnierpy.com
    """
    sniper.init()


def cli():
    try:
        app()
    except Exception as ex:
        tb = ex.__traceback__
        sniper.log.error(f"type: {type(ex).__name__}")
        sniper.log.error(f"message: {str(ex)}")
        while tb is not None:
            sniper.log.error(f"{tb.tb_frame.f_code.co_filename}:{tb.tb_lineno}")
            tb = tb.tb_next
    finally:
        sniper.on_shutdown()


if __name__ == '__name__':
    cli()



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')


setup(
    name="MCsniperPY",
    version='0.18',
    description='Minecraft name sniper written in Python',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/MCsniperPY/MCsniperPY',
    author='Kqzz',
    license='MIT',
    packages=['mcsniperpy', 'mcsniperpy.util', 'mcsniperpy.util.classes'],
    install_requires=[
        'typer',
        'aiohttp',
        'colorama',
        'bs4'
    ],
    entry_points={
        'console_scripts': ['mcsniperpy=mcsniperpy.cli:cli']
    },
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9'
    ],
)
